/////////////////////////////////////////////////////////////////
//
//	Simple Transform Feedback Example
//	Author: Shiben Bhattacharjee
//	shiben [at] research [dot] iiit [dot] ac [dot] in
//	http://cvit.iiit.ac.in/
//
/////////////////////////////////////////////////////////////////

#include<stdio.h>
#include<stdlib.h>

#ifdef _WIN32
#include<GL/glee.h>
#else
#define GL_GLEXT_PROTOTYPES
#endif

#include<GL/glut.h>
#include<GL/glu.h>
#include<GL/gl.h>

GLuint tfvbo;
int count=2;
float t=0;

void setupTFvbo(void)
{
	//position to be copied in the feedback buffer, 4 components, 3rd option is optional in position's case
	GLint att[3]= {GL_POSITION, 4, 0};

	//generating the buffer, note that GL_TRANSFORM_FEEDBACK_BUFFER_NV is NOT a buffer type
	glGenBuffersARB(1,&tfvbo);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,tfvbo);
	glBufferDataARB(GL_ARRAY_BUFFER_ARB,count*4*sizeof(float),0,GL_STATIC_DRAW_ARB);

	//bind the tfvbo to get the feedback
	glBindBufferOffsetNV(GL_TRANSFORM_FEEDBACK_BUFFER_NV,0,tfvbo,0);

	//record position attrib (no shaders, so cannot feedback varyings)
	glTransformFeedbackAttribsNV(1,att,GL_INTERLEAVED_ATTRIBS_NV);
}
void init(void)
{
	glClearColor(0.5,0.5,0.5,1);
	glPointSize(5);
	setupTFvbo();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1,1,-1,1,0.2,2);
	glMatrixMode(GL_MODELVIEW);
}
void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glRotatef(t+=0.001,0,0,1);

//------display the normal rendered object at this viewport, with color red
	glColor3f(1,0,0);
	glViewport(0,0,256,256);

	//start transform feedback so that vertices get targetted to 'tfvbo'
	glBeginTransformFeedbackNV(GL_POINTS);
	//render primitives and copy these positions to tranform feedback buffer
	glBegin( GL_POINTS );
		glVertex4f( -0.5, -0.5, -0.2, 1 );
		glVertex4f(  0.5,  0.5, -0.2, 1 );
	glEnd();
	//end transform feedback
	glEndTransformFeedbackNV();

//------display the transform feedback recorded data to this viewport
	glColor3f(0,1,0);
	glViewport(256,0,256,256);

	//render the buffer which has recieved data from transform feedback
	//note that glRotate is still active, and it has already rotated once
	//means it will rotate twice the speed
	glEnableClientState(GL_VERTEX_ARRAY);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB,tfvbo);
	glVertexPointer(4,GL_FLOAT,0,NULL);
	glDrawArrays(GL_POINTS,0,count);
	glDisableClientState(GL_VERTEX_ARRAY);

	glutSwapBuffers();
}
int main(int argc,char **argv)
{
	glutInit( &argc, argv );
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
	glutInitWindowSize(512,256);
	glutCreateWindow("Transform Feedback");
	glutDisplayFunc( display );
	glutIdleFunc( display );
	init();
	glutMainLoop();
	return 0;
}
